using UnityEngine;
using UnityEngine.U2D;
using System.Collections;
using System.Collections.Generic;

namespace Mod
{
    public class Mod
    {
        public static void Main()
        {
            CategoryBuilder.Create("Weather", "Weather items!", ModAPI.LoadSprite("icon.png"));

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Generator"),
                    NameOverride = "Tornado",
                    DescriptionOverride = "Harness the might of a thundering tornado!",
                    CategoryOverride = ModAPI.FindCategory("Weather"),
                    ThumbnailOverride = ModAPI.LoadSprite("blanket.png"), //Doesn't exist yet!
                    AfterSpawn = (Instance) =>
                    {
                        Instance.GetComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("blanket.png"); //Doesn't exist yet!
                        Tornado tornado = Instance.AddComponent<Tornado>();

                        int length = 40;

                        Transform[] pivots = new Transform[length];
                        float[] counter = new float[length];

                        for (int i = 0; i < pivots.Length; i++)
                        {
                            float y = Mathf.Sin(i);
                            pivots[i] = new GameObject("pivot " + i).transform;
                            //waypoints[i].SetParent(transform);

                            pivots[i].gameObject.AddComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("dust.png");
                            counter[i] = i;

                            pivots[i].SetParent(Instance.transform);
                            pivots[i].transform.localScale = Vector3.zero;
                        }

                        tornado.pivots = pivots;
                        tornado.counter = counter;
                        tornado.clips = ModAPI.LoadSound("Wind.wav");
                    }
                }
            );
        }
    }

    class Tornado : MonoBehaviour
    {
        [SkipSerialisation]
        public AudioSource audio;
        [SkipSerialisation]
        public AudioClip clips;

        public float rps = -5;
        public float width = 0;
        public float height = 0;
        public float size = 0;
        public float negate;

        public float volume;

        public int length = 40;

        public Color color = new Color(1, 1, 1, 0.75f);


        public Transform[] pivots;
        public float[] counter;

        public bool active;

        void Awake()
        {
            audio = gameObject.AddComponent<AudioSource>();
            audio.outputAudioMixerGroup = Global.main.SoundEffects;
            audio.gameObject.AddComponent<AudioSourceTimeScaleBehaviour>();
        }

        void Update()
        {
            IsActive();

            negate = active ? 1 : -1;

            audio.volume = volume; //This is just for variety, it's not required.
            if (active && !audio.isPlaying) 
            { 
                audio.clip = clips;

                audio.Play();

                audio.loop = true;
            }
        }

        // Update is called once per frame
        void Use()
        {
            active = !active;
        }

        void IsActive()
        { 
            height += Time.deltaTime * negate;
            size += Time.deltaTime * 5 * negate;
            width += Time.deltaTime * 1.5f * negate;
            volume += Time.deltaTime / 2 * negate;

            height = Mathf.Clamp(height, 0, 2);
            size = Mathf.Clamp(size, 0, 10);
            width = Mathf.Clamp(width, 0, 5);
            volume = Mathf.Clamp(volume, 0, 2);

            for (int i = 0; i < pivots.Length; i++)
            {
                float y = Mathf.Sin(counter[i] * 1);
                //float curve = y * Mathf.Sin(i * bend);
                pivots[i].position = new Vector3(transform.position.x + (y * width * (i * 0.1f)), transform.position.y + (i * height), transform.position.z);
                counter[i] += Time.deltaTime * rps;

                pivots[i].localScale = Vector3.one * size;

                pivots[i].GetComponent<SpriteRenderer>().color = color;

                pivots[i].rotation = Quaternion.identity;
            }

            if(active) Collide();
        }

        void Collide()
        {

            Vector3 target = new Vector3(transform.position.x,transform.position.y + (height * length), 0);
            Vector3 initialDist = target - transform.position;
            Vector3 halfExtents = new Vector3(width * (length / 4), 1, width * (length / 4));

            Vector3 current = transform.position;
            var direction = target - current;
            var angle = Mathf.Atan2(direction.y, direction.x) * Mathf.Rad2Deg;

            float distance = Vector3.Distance((transform.position + new Vector3(0, height * length, 0)), transform.position);


            RaycastHit2D[] collider = Physics2D.BoxCastAll(transform.position, halfExtents, 0, initialDist,
                distance);

            if (collider.Length > 0)
            {
                //color = new Color(0, 1, 0, 0.75f);

                foreach (RaycastHit2D hit in collider)
                {
                    Rigidbody2D rb = hit.transform.gameObject.GetComponent<Rigidbody2D>();

                    if (rb != null)
                    {
                        rb.velocity += new Vector2(transform.position.x - hit.transform.position.x,2) * 20 * Time.deltaTime;
                    }
                }
            }
            //else
            //{
            //    color = new Color(1, 0, 0, 0.75f);
            //}
        }
    }
}